@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2017, 2018 Laboratoire de Recherche et Développement de
+// Copyright (C) 2017, 2018, 2019 Laboratoire de Recherche et Développement de
 // l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -45,12 +45,9 @@
 #include <spot/twaalgos/hoa.hh>
 #include <spot/twaalgos/postproc.hh>
 #include <spot/twaalgos/isdet.hh>
-#include <spot/twaalgos/determinize.hh>
-#include <spot/twaalgos/dualize.hh>
+#include <spot/twaalgos/complement.hh>
 #include <spot/twaalgos/alternation.hh>
 #include <spot/twaalgos/cleanacc.hh>
-#include <spot/twaalgos/remfin.hh>
-#include <spot/twaalgos/product.hh>
 #include <spot/misc/escape.hh>
 #include <spot/misc/timer.hh>
 
@@ -116,10 +113,12 @@ static const argp_option options[] =
       "all available optimizations (slow, default)", 0 },
     /**************************************************/
     { nullptr, 0, nullptr, 0, "Output options:", -15 },
+    { "quiet", 'q', nullptr, 0,
+      "suppress all normal output in absence of errors", 0 },
     { "save-bogus", OPT_BOGUS, "[>>]FILENAME", 0,
-      "save formulas for which problems were detected in FILENAME", -1 },
+      "save formulas for which problems were detected in FILENAME", 0 },
     { "verbose", OPT_VERBOSE, nullptr, 0,
-      "print what is being done, for debugging", -1 },
+      "print what is being done, for debugging", 0 },
     { nullptr, 0, nullptr, 0,
       "If an output FILENAME is prefixed with '>>', it is open "
       "in append mode instead of being truncated.", -14 },
@@ -138,6 +137,7 @@ const struct argp_child children[] =
   };
 
 static bool verbose = false;
+static bool quiet = false;
 static bool ignore_exec_fail = false;
 static unsigned ignored_exec_fail = 0;
 static bool fail_on_timeout = false;
@@ -158,6 +158,10 @@ parse_opt(int key, char* arg, struct argp_state*)
     case 'F':
       jobs.emplace_back(arg, true);
       break;
+    case 'q':
+      quiet = true;
+      verbose = false;
+      break;
     case OPT_BOGUS:
       {
         bogus_output = new output_file(arg);
@@ -199,6 +203,7 @@ parse_opt(int key, char* arg, struct argp_state*)
       break;
     case OPT_VERBOSE:
       verbose = true;
+      quiet = false;
       break;
     case ARGP_KEY_ARG:
       if (arg[0] == '-' && !arg[1])
@@ -388,8 +393,12 @@ namespace
       format(command, tools[tool_num].cmd);
 
       std::string cmd = command.str();
-      std::cerr << "Running [" << l << tool_num << "]: "
-                << cmd << std::endl;
+      auto disp_cmd = [&]() {
+                        std::cerr << "Running [" << l << tool_num
+                                  << "]: " << cmd << '\n';
+                      };
+      if (!quiet)
+        disp_cmd();
       spot::process_timer timer;
       timer.start();
       int es = exec_with_timeout(cmd.c_str());
@@ -401,12 +410,15 @@ namespace
         {
           if (fail_on_timeout)
             {
+              if (quiet)
+                disp_cmd();
               global_error() << "error: timeout during execution of command\n";
               end_error();
             }
           else
             {
-              std::cerr << "warning: timeout during execution of command\n";
+              if (!quiet)
+                std::cerr << "warning: timeout during execution of command\n";
               ++timeout_count;
             }
           status_str = "timeout";
@@ -415,6 +427,8 @@ namespace
         }
       else if (WIFSIGNALED(es))
         {
+          if (quiet)
+            disp_cmd();
           status_str = "signal";
           problem = true;
           es = WTERMSIG(es);
@@ -428,6 +442,8 @@ namespace
           status_str = "exit code";
           if (!ignore_exec_fail)
             {
+              if (quiet)
+                disp_cmd();
               problem = true;
               global_error() << "error: execution returned exit code "
                              << es << ".\n";
@@ -436,8 +452,9 @@ namespace
           else
             {
               problem = false;
-              std::cerr << "warning: execution returned exit code "
-                        << es << ".\n";
+              if (!quiet)
+                std::cerr << "warning: execution returned exit code "
+                          << es << ".\n";
               ++ignored_exec_fail;
             }
         }
@@ -452,6 +469,8 @@ namespace
                                      opt_parse);
           if (!aut->errors.empty())
             {
+              if (quiet)
+                disp_cmd();
               status_str = "parse error";
               problem = true;
               es = -1;
@@ -463,6 +482,8 @@ namespace
             }
           else if (aut->aborted)
             {
+              if (quiet)
+                disp_cmd();
               status_str = "aborted";
               problem = true;
               es = -1;
@@ -512,19 +533,18 @@ namespace
     if (aut_i->num_sets() + aut_j->num_sets() >
         spot::acc_cond::mark_t::max_accsets())
       {
-        std::cerr << "info: building " << autname(i)
-                  << '*' << autname(j, true)
-                  << " requires more acceptance sets than supported\n";
+        if (!quiet)
+          std::cerr << "info: building " << autname(i)
+                    << '*' << autname(j, true)
+                    << " requires more acceptance sets than supported\n";
         return false;
       }
 
-    auto prod = spot::product(aut_i, aut_j);
-
     if (verbose)
       std::cerr << "info: check_empty "
                 << autname(i) << '*' << autname(j, true) << '\n';
 
-    auto w = prod->accepting_word();
+    auto w = aut_i->intersecting_word(aut_j);
     if (w)
       {
         std::ostream& err = global_error();
@@ -579,14 +599,21 @@ namespace
 
       input_statistics.push_back(in_statistics());
 
-      std::cerr << bold << source << reset_color;
       input_statistics[round_num].input_source = std::move(source);
       if (auto name = input->get_named_prop<std::string>("automaton-name"))
-        {
-          std::cerr << '\t' << *name;
-          input_statistics[round_num].input_name = *name;
-        }
-      std::cerr << '\n';
+        input_statistics[round_num].input_name = *name;
+
+      auto disp_src = [&]() {
+                        std::cerr << bold
+                                  << input_statistics[round_num].input_source
+                                  << reset_color;
+                        if (!input_statistics[round_num].input_name.empty())
+                          std::cerr << '\t'
+                                    << input_statistics[round_num].input_name;
+                        std::cerr << '\n';
+                      };
+      if (!quiet)
+        disp_src();
       input_statistics[round_num].input.set(input);
 
       int problems = 0;
@@ -612,7 +639,6 @@ namespace
           problems += prob;
         }
       spot::cleanup_tmpfiles();
-      ++round_num;
       output_statistics.push_back(std::move(stats));
 
       if (verbose)
@@ -628,15 +654,16 @@ namespace
 
       if (!no_checks)
         {
-          std::cerr << "Performing sanity checks and gathering statistics..."
-                    << std::endl;
-
+          if (!quiet)
+            std::cerr
+              << "Performing sanity checks and gathering statistics...\n";
           {
             bool print_first = true;
             for (unsigned i = 0; i < mi; ++i)
               {
                 if (!pos[i])
                   continue;
+                cleanup_acceptance_here(pos[i]);
                 if (!pos[i]->is_existential())
                   {
                     if (verbose)
@@ -659,8 +686,6 @@ namespace
                         std::cerr << '\n';
                       }
                   }
-                if (is_universal(pos[i]))
-                  neg[i] = dualize(pos[i]);
               }
           }
 
@@ -668,19 +693,14 @@ namespace
             bool print_first = verbose;
             for (unsigned i = 0; i < mi; ++i)
               {
-                if (pos[i] && !neg[i])
+                if (pos[i])
                   {
                     if (print_first)
                       {
-                        std::cerr << "info: complementing non-deterministic "
-                        "automata via determinization...\n";
+                        std::cerr << "info: complementing automata...\n";
                         print_first = false;
                       }
-                    spot::postprocessor p;
-                    p.set_type(spot::postprocessor::Generic);
-                    p.set_pref(spot::postprocessor::Deterministic);
-                    p.set_level(level);
-                    neg[i] = dualize(p.run(pos[i]));
+                    neg[i] = spot::complement(pos[i]);
                     if (verbose)
                       {
                         std::cerr << "info:   "
@@ -690,54 +710,11 @@ namespace
                         printsize(neg[i], false);
                         std::cerr << '\t' << autname(i, true) << '\n';
                       }
+                    cleanup_acceptance_here(pos[i]);
                   }
               };
           }
 
-          {
-            bool print_first = true;
-            auto tmp = [&](std::vector<spot::twa_graph_ptr>& x, unsigned i,
-                           bool neg)
-              {
-                if (!x[i])
-                  return;
-                if (x[i]->acc().uses_fin_acceptance())
-                  {
-                    if (verbose)
-                      {
-                        if (print_first)
-                          {
-                            std::cerr <<
-                              "info: getting rid of any Fin acceptance...\n";
-                            print_first = false;
-                          }
-                        std::cerr << "info:   "
-                                  << std::setw(8) << autname(i, neg) << '\t';
-                        printsize(x[i], false);
-                        std::cerr << " ->";
-                      }
-                    cleanup_acceptance_here(x[i]);
-                    x[i] = remove_fin(x[i]);
-                    if (verbose)
-                      {
-                        std::cerr << ' ';
-                        printsize(x[i], false);
-                        std::cerr << '\n';
-                      }
-                  }
-                else
-                  {
-                    // Remove useless sets nonetheless.
-                    cleanup_acceptance_here(x[i]);
-                  }
-              };
-            for (unsigned i = 0; i < mi; ++i)
-              {
-                tmp(pos, i, false);
-                tmp(neg, i, true);
-              }
-          }
-
           // Just make a circular implication check
           // A0 <= A1, A1 <= A2, ..., AN <= A0
           unsigned ok = 0;
@@ -766,15 +743,23 @@ namespace
         }
       else
         {
-          std::cerr << "Gathering statistics..." << std::endl;
+          if (!quiet)
+            std::cerr << "Gathering statistics...\n";
         }
 
 
       if (problems && bogus_output)
         print_hoa(bogus_output->ostream(), input) << std::endl;
 
-      std::cerr << std::endl;
+      if (problems && quiet)
+        {
+          std::cerr << "input automaton was ";
+          disp_src();
+        }
+      if (!quiet || problems)
+        std::cerr << '\n';
 
+      ++round_num;
       // Shall we stop processing now?
       abort_run = global_error_flag && stop_on_error;
       return problems;
@@ -851,7 +836,7 @@ main(int argc, char** argv)
         {
           error(2, 0, "no automaton to translate");
         }
-      else
+      else if (!quiet)
         {
           if (global_error_flag)
             {
@@ -860,21 +845,20 @@ main(int argc, char** argv)
                 err << ("error: some error was detected during the above "
                         "runs.\n       Check file ")
                     << bogus_output_filename
-                    << " for problematic automata.";
+                    << " for problematic automata.\n";
               else
                 err << ("error: some error was detected during the above "
                         "runs,\n       please search for 'error:' messages"
-                        " in the above trace.");
-              err << std::endl;
+                        " in the above trace.\n");
               end_error();
             }
           else if (timeout_count == 0 && ignored_exec_fail == 0)
             {
-              std::cerr << "No problem detected." << std::endl;
+              std::cerr << "No problem detected.\n";
             }
           else
             {
-              std::cerr << "No major problem detected." << std::endl;
+              std::cerr << "No major problem detected.\n";
             }
 
           unsigned additional_errors = 0U;
@@ -905,7 +889,7 @@ main(int argc, char** argv)
                 }
               if (additional_errors == 1)
                 std::cerr << '.';
-              std::cerr << std::endl;
+              std::cerr << '\n';
             }
         }
 
