@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2013-2018 Laboratoire de Recherche et Développement
+// Copyright (C) 2013-2019 Laboratoire de Recherche et Développement
 // de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -49,6 +49,7 @@
 #include <spot/twaalgos/canonicalize.hh>
 #include <spot/twaalgos/cobuchi.hh>
 #include <spot/twaalgos/cleanacc.hh>
+#include <spot/twaalgos/complement.hh>
 #include <spot/twaalgos/contains.hh>
 #include <spot/twaalgos/degen.hh>
 #include <spot/twaalgos/dtwasat.hh>
@@ -110,6 +111,7 @@ enum {
   OPT_HIGHLIGHT_NONDET_EDGES,
   OPT_HIGHLIGHT_NONDET_STATES,
   OPT_HIGHLIGHT_WORD,
+  OPT_HIGHLIGHT_ACCEPTING_RUN,
   OPT_HIGHLIGHT_LANGUAGES,
   OPT_INSTUT,
   OPT_INCLUDED_IN,
@@ -317,7 +319,7 @@ static const argp_option options[] =
     { "cleanup-acceptance", OPT_CLEAN_ACC, nullptr, 0,
       "remove unused acceptance sets from the automaton", 0 },
     { "complement", OPT_COMPLEMENT, nullptr, 0,
-      "complement each automaton (currently via determinization)", 0 },
+      "complement each automaton (different strategies are used)", 0 },
     { "complement-acceptance", OPT_COMPLEMENT_ACC, nullptr, 0,
       "complement the acceptance condition (without touching the automaton)",
       0 },
@@ -373,6 +375,8 @@ static const argp_option options[] =
       "(see spot-x)."
       , 0 },
     { nullptr, 0, nullptr, 0, "Decorations (for -d and -H1.1 output):", 9 },
+    { "highlight-accepting-run", OPT_HIGHLIGHT_ACCEPTING_RUN, "NUM",
+      OPTION_ARG_OPTIONAL, "highlight one accepting run using color NUM", 0},
     { "highlight-nondet-states", OPT_HIGHLIGHT_NONDET_STATES, "NUM",
       OPTION_ARG_OPTIONAL, "highlight nondeterministic states with color NUM",
       0 },
@@ -628,6 +632,7 @@ static bool opt_split_edges = false;
 static const char* opt_sat_minimize = nullptr;
 static int opt_highlight_nondet_states = -1;
 static int opt_highlight_nondet_edges = -1;
+static int opt_highlight_accepting_run = -1;
 static bool opt_highlight_languages = false;
 static bool opt_dca = false;
 static bool opt_streett_like = false;
@@ -764,7 +769,7 @@ parse_opt(int key, char* arg, struct argp_state*)
                     {
                       std::cerr << ", '" << acc_is_args[i] << '\'';
                     }
-                std::cerr << std::endl;
+                std::cerr << '\n';
                 exit(2);
               }
           }
@@ -844,6 +849,10 @@ parse_opt(int key, char* arg, struct argp_state*)
     case OPT_HAS_UNIV_BRANCHING:
       opt_has_univ_branching = true;
       break;
+    case OPT_HIGHLIGHT_ACCEPTING_RUN:
+      opt_highlight_accepting_run =
+        arg ? to_pos_int(arg, "--highlight-accepting-run") : 1;
+      break;
     case OPT_HIGHLIGHT_NONDET:
       {
         int v = arg ? to_pos_int(arg, "--highlight-nondet") : 1;
@@ -1492,7 +1501,7 @@ namespace
 
       if (opt_complement)
         {
-          aut = spot::dualize(ensure_deterministic(aut));
+          aut = spot::complement(aut);
           aut->merge_edges();
         }
 
@@ -1528,6 +1537,9 @@ namespace
       if (opt_highlight_languages)
         spot::highlight_languages(aut);
 
+      if (opt_highlight_accepting_run >= 0)
+        aut->accepting_run()->highlight(opt_highlight_accepting_run);
+
       if (!opt->hl_words.empty())
         for (auto& word_aut: opt->hl_words)
           {
