@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2012-2018 Laboratoire de Recherche et Développement
+// Copyright (C) 2012-2019 Laboratoire de Recherche et Développement
 // de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -55,7 +55,7 @@
 #include <spot/twaalgos/sccinfo.hh>
 #include <spot/twaalgos/isweakscc.hh>
 #include <spot/twaalgos/word.hh>
-#include <spot/twaalgos/dualize.hh>
+#include <spot/twaalgos/complement.hh>
 #include <spot/twaalgos/cleanacc.hh>
 #include <spot/twaalgos/alternation.hh>
 #include <spot/misc/formater.hh>
@@ -86,6 +86,8 @@ enum {
   OPT_BOGUS,
   OPT_CSV,
   OPT_DENSITY,
+  OPT_DET_MAX_STATES,
+  OPT_DET_MAX_EDGES,
   OPT_DUPS,
   OPT_FAIL_ON_TIMEOUT,
   OPT_GRIND,
@@ -118,8 +120,18 @@ static const argp_option options[] =
     { "no-complement", OPT_NOCOMP, nullptr, 0,
       "do not complement deterministic automata to perform extra checks", 0 },
     { "determinize", 'D', nullptr, 0,
-      "determinize non-deterministic automata so that they"
+      "always determinize non-deterministic automata so that they"
       "can be complemented; also implicitly sets --products=0", 0 },
+    { "determinize-max-states", OPT_DET_MAX_STATES, "N", 0,
+      "attempt to determinize non-deterministic automata so they can be "
+      "complemented, unless the deterministic automaton would have more "
+      "than N states.  Without this option or -D, determinizations "
+      "are attempted up to 500 states.", 0 },
+    { "determinize-max-edges", OPT_DET_MAX_EDGES, "N", 0,
+      "attempt to determinize non-deterministic automata so they can be "
+      "complemented, unless the deterministic automaton would have more "
+      "than N edges.  Without this option or -D, determinizations "
+      "are attempted up to 5000 edges.", 0 },
     { "stop-on-error", OPT_STOP_ERR, nullptr, 0,
       "stop on first execution error or failure to pass"
       " sanity checks (timeouts are OK)", 0 },
@@ -164,6 +176,8 @@ static const argp_option options[] =
     { "grind", OPT_GRIND, "[>>]FILENAME", 0,
       "for each formula for which a problem was detected, write a simpler " \
       "formula that fails on the same test in FILENAME", 0 },
+    { "quiet", 'q', nullptr, 0,
+      "suppress all normal output in absence of error", 0 },
     { "save-bogus", OPT_BOGUS, "[>>]FILENAME", 0,
       "save formulas for which problems were detected in FILENAME", 0 },
     { "verbose", OPT_VERBOSE, nullptr, 0,
@@ -195,6 +209,10 @@ static bool allow_dups = false;
 static bool no_checks = false;
 static bool no_complement = false;
 static bool determinize = false;
+static bool max_det_states_given = false;
+static bool max_det_edges_given = false;
+static unsigned max_det_states = 500U;
+static unsigned max_det_edges = 5000U;
 static bool stop_on_error = false;
 static int seed = 0;
 static unsigned products = 1;
@@ -204,6 +222,7 @@ static const char* bogus_output_filename = nullptr;
 static output_file* bogus_output = nullptr;
 static output_file* grind_output = nullptr;
 static bool verbose = false;
+static bool quiet = false;
 static bool ignore_exec_fail = false;
 static unsigned ignored_exec_fail = 0;
 static bool fail_on_timeout = false;
@@ -427,6 +446,32 @@ parse_opt(int key, char* arg, struct argp_state*)
     case 'D':
       determinize = true;
       products = 0;
+      max_det_states = -1U;
+      max_det_edges = -1U;
+      if (max_det_states_given)
+        error(2, 0, "Options --determinize-max-states and "
+              "--determinize are incompatible.");
+      if (max_det_edges_given)
+        error(2, 0, "Options --determinize-max-edges and "
+              "--determinize are incompatible.");
+      break;
+    case 'q':
+      verbose = false;
+      quiet = true;
+      break;
+    case OPT_DET_MAX_EDGES:
+      max_det_edges_given = true;
+      max_det_states = to_pos_int(arg, "--determinize-max-edges");
+      if (determinize)
+        error(2, 0, "Options --determinize-max-edges and "
+              "--determinize are incompatible.");
+      break;
+    case OPT_DET_MAX_STATES:
+      max_det_states_given = true;
+      max_det_states = to_pos_int(arg, "--determinize-max-states");
+      if (determinize)
+        error(2, 0, "Options --determinize-max-states and "
+              "--determinize are incompatible.");
       break;
     case ARGP_KEY_ARG:
       if (arg[0] == '-' && !arg[1])
@@ -434,6 +479,9 @@ parse_opt(int key, char* arg, struct argp_state*)
       else
         tools_push_trans(arg);
       break;
+    case OPT_AMBIGUOUS:
+      opt_ambiguous = true;
+      break;
     case OPT_AUTOMATA:
       opt_automata = true;
       break;
@@ -466,16 +514,6 @@ parse_opt(int key, char* arg, struct argp_state*)
       want_stats = true;
       json_output = arg ? arg : "-";
       break;
-    case OPT_PRODUCTS:
-      if (*arg == '+')
-        {
-          products_avg = false;
-          ++arg;
-        }
-      products = to_pos_int(arg, "--products");
-      if (products == 0)
-        products_avg = false;
-      break;
     case OPT_NOCHECKS:
       no_checks = true;
       no_complement = true;
@@ -486,6 +524,16 @@ parse_opt(int key, char* arg, struct argp_state*)
     case OPT_OMIT:
       opt_omit = true;
       break;
+    case OPT_PRODUCTS:
+      if (*arg == '+')
+        {
+          products_avg = false;
+          ++arg;
+        }
+      products = to_pos_int(arg, "--products");
+      if (products == 0)
+        products_avg = false;
+      break;
     case OPT_REFERENCE:
       tools_push_trans(arg, true);
       break;
@@ -501,11 +549,9 @@ parse_opt(int key, char* arg, struct argp_state*)
     case OPT_STRENGTH:
       opt_strength = true;
       break;
-    case OPT_AMBIGUOUS:
-      opt_ambiguous = true;
-      break;
     case OPT_VERBOSE:
       verbose = true;
+      quiet = false;
       break;
     default:
       return ARGP_ERR_UNKNOWN;
@@ -533,8 +579,12 @@ namespace
       format(command, tools[translator_num].cmd);
 
       std::string cmd = command.str();
-      std::cerr << "Running [" << l << translator_num << "]: "
-                << cmd << std::endl;
+      auto disp_cmd = [&]() {
+                        std::cerr << "Running [" << l << translator_num
+                                  << "]: " << cmd << '\n';
+                      };
+      if (!quiet)
+        disp_cmd();
       spot::process_timer timer;
       timer.start();
       int es = exec_with_timeout(cmd.c_str());
@@ -546,12 +596,15 @@ namespace
         {
           if (fail_on_timeout)
             {
+              if (quiet)
+                disp_cmd();
               global_error() << "error: timeout during execution of command\n";
               end_error();
             }
           else
             {
-              std::cerr << "warning: timeout during execution of command\n";
+              if (!quiet)
+                std::cerr << "warning: timeout during execution of command\n";
               ++timeout_count;
             }
           status_str = "timeout";
@@ -560,6 +613,8 @@ namespace
         }
       else if (WIFSIGNALED(es))
         {
+          if (quiet)
+            disp_cmd();
           status_str = "signal";
           problem = true;
           es = WTERMSIG(es);
@@ -573,6 +628,8 @@ namespace
           status_str = "exit code";
           if (!ignore_exec_fail)
             {
+              if (quiet)
+                disp_cmd();
               problem = true;
               global_error() << "error: execution returned exit code "
                              << es << ".\n";
@@ -581,8 +638,9 @@ namespace
           else
             {
               problem = false;
-              std::cerr << "warning: execution returned exit code "
-                        << es << ".\n";
+              if (!quiet)
+                std::cerr << "warning: execution returned exit code "
+                          << es << ".\n";
               ++ignored_exec_fail;
             }
         }
@@ -597,6 +655,8 @@ namespace
                                      opt_parse);
           if (!aut->errors.empty())
             {
+              if (quiet)
+                disp_cmd();
               status_str = "parse error";
               problem = true;
               es = -1;
@@ -608,6 +668,8 @@ namespace
             }
           else if (aut->aborted)
             {
+              if (quiet)
+                disp_cmd();
               status_str = "aborted";
               problem = true;
               es = -1;
@@ -698,16 +760,19 @@ namespace
         // complemented, or the --verbose option is used,
         if (!verbose && (icomp || jcomp))
           return false;
-        std::cerr << "info: building ";
-        if (icomp)
-          std::cerr << "Comp(N" << i << ')';
-        else
-          std::cerr << 'P' << i;
-        if (jcomp)
-          std::cerr << "*Comp(P" << j << ')';
-        else
-          std::cerr << "*N" << j;
-        std::cerr << " requires more acceptance sets than supported\n";
+        if (!quiet)
+          {
+            std::cerr << "info: building ";
+            if (icomp)
+              std::cerr << "Comp(N" << i << ')';
+            else
+              std::cerr << 'P' << i;
+            if (jcomp)
+              std::cerr << "*Comp(P" << j << ')';
+            else
+              std::cerr << "*N" << j;
+            std::cerr << " requires more acceptance sets than supported\n";
+          }
         return false;
       }
 
@@ -894,16 +959,18 @@ namespace
           unsigned mutation_max;
           while        (res)
             {
-              std::cerr << "Trying to find a bogus mutation of " << bold
-                        << bogus << reset_color << "...\n";
+              if (!quiet)
+                std::cerr << "Trying to find a bogus mutation of " << bold
+                          << bogus << reset_color << "...\n";
               mutations = mutate(f);
               mutation_count = 1;
               mutation_max = mutations.size();
               res = 0;
               for (auto g: mutations)
                 {
-                  std::cerr << "Mutation " << mutation_count << '/'
-                            << mutation_max << ": ";
+                  if (!quiet)
+                    std::cerr << "Mutation " << mutation_count << '/'
+                              << mutation_max << ": ";
                   f = g;
                   res = process_formula(g);
                   if (res)
@@ -920,9 +987,10 @@ namespace
                     bogus_output->ostream() << bogus << std::endl;
                 }
             }
-          std::cerr << "Smallest bogus mutation found for "
-                    << bold << input << reset_color << " is "
-                    << bold << bogus << reset_color << ".\n\n";
+          if (!quiet)
+            std::cerr << "Smallest bogus mutation found for "
+                      << bold << input << reset_color << " is "
+                      << bold << bogus << reset_color << ".\n\n";
           grind_output->ostream() << bogus << std::endl;
         }
       return 0;
@@ -977,24 +1045,27 @@ namespace
       // Call formula() before printing anything else, in case it
       // complains.
       std::string fstr = runner.formula();
-      if (filename)
-        std::cerr << filename << ':';
-      if (linenum)
-        std::cerr << linenum << ':';
-      if (filename || linenum)
-        std::cerr << ' ';
-      std::cerr << bold << fstr << reset_color << '\n';
+      if (!quiet)
+        {
+          if (filename)
+            std::cerr << filename << ':';
+          if (linenum)
+            std::cerr << linenum << ':';
+          if (filename || linenum)
+            std::cerr << ' ';
+          std::cerr << bold << fstr << reset_color << '\n';
+        }
 
       // Make sure we do not translate the same formula twice.
       if (!allow_dups)
         {
           if (!unique_set.insert(f).second)
             {
-              std::cerr
-                << ("warning: This formula or its negation has already"
-                    " been checked.\n         Use --allow-dups if it "
-                    "should not be ignored.\n")
-                << std::endl;
+              if (!quiet)
+                std::cerr
+                  << ("warning: This formula or its negation has already"
+                      " been checked.\n         Use --allow-dups if it "
+                      "should not be ignored.\n\n");
               return 0;
             }
         }
@@ -1094,10 +1165,13 @@ namespace
             }
         }
 
+      bool missing_complement = true;
+
       if (!no_checks)
         {
-          std::cerr << "Performing sanity checks and gathering statistics..."
-                    << std::endl;
+          if (!quiet)
+            std::cerr
+              << "Performing sanity checks and gathering statistics...\n";
 
           // If we have reference tools, pick the smallest of their
           // automata for positive and negative references.
@@ -1122,7 +1196,7 @@ namespace
               return smallest_ref;
             };
 
-          // This are not our definitive choice for reference
+          // These are not our definitive choice for reference
           // automata, because the sizes might change after we remove
           // alternation and Fin acceptance.  But we need to know now
           // if we will have a pair of reference automata in order to
@@ -1158,32 +1232,27 @@ namespace
                   std::cerr << ")\n";
                 }
             };
-          auto complement = [&](const std::vector<spot::twa_graph_ptr>& x,
-                                std::vector<spot::twa_graph_ptr>& comp,
-                                unsigned i)
-            {
-              if (!no_complement && x[i] && is_universal(x[i]))
-                comp[i] = dualize(x[i]);
-            };
 
+          // Remove alternation
           for (size_t i = 0; i < m; ++i)
             {
               unalt(pos, i, 'P');
               unalt(neg, i, 'N');
-              // Do not complement reference automata if we have a
-              // reference pair.
-              if (smallest_pos_ref >= 0 && tools[i].reference)
-                continue;
-              complement(pos, comp_pos, i);
-              complement(neg, comp_neg, i);
             }
 
-          if (determinize && !no_complement)
+          // Complement
+          if (!no_complement)
             {
+              spot::output_aborter aborter_(max_det_states,
+                                            max_det_edges);
+              spot::output_aborter* aborter = nullptr;
+              if (max_det_states != -1U || max_det_edges != -1U)
+                aborter = &aborter_;
+
               print_first = verbose;
-              auto tmp = [&](std::vector<spot::twa_graph_ptr>& from,
-                             std::vector<spot::twa_graph_ptr>& to, int i,
-                             char prefix)
+              auto comp = [&](std::vector<spot::twa_graph_ptr>& from,
+                              std::vector<spot::twa_graph_ptr>& to, int i,
+                              char prefix)
                 {
                   if (from[i] && !to[i])
                     {
@@ -1193,29 +1262,41 @@ namespace
                         return;
                       if (print_first)
                         {
-                          std::cerr << "info: complementing non-deterministic "
-                            "automata via determinization...\n";
+                          std::cerr << "info: complementing automata...\n";
                           print_first = false;
                         }
-                      spot::postprocessor p;
-                      p.set_type(spot::postprocessor::Generic);
-                      p.set_pref(spot::postprocessor::Deterministic);
-                      p.set_level(spot::postprocessor::Low);
-                      to[i] = dualize(p.run(from[i]));
+                      if (verbose)
+                        std::cerr << "info:   " << prefix << i;
+                      if (aborter && aborter->too_large(from[i])
+                          && !spot::is_universal(from[i]))
+                        missing_complement = true;
+                      else
+                        to[i] = spot::complement(from[i], aborter);
                       if (verbose)
                         {
-                          std::cerr << "info:   " << prefix << i << "\t(";
-                          printsize(from[i]);
-                          std::cerr << ") -> (";
-                          printsize(to[i]);
-                          std::cerr << ")\tComp(" << prefix << i << ")\n";
+                          if (to[i])
+                            {
+                              std::cerr << "\t(";
+                              printsize(from[i]);
+                              std::cerr << ") -> (";
+                              printsize(to[i]);
+                              std::cerr << ")\tComp(" << prefix << i << ")\n";
+                            }
+                          else
+                            {
+                              std::cerr << "\tnot complemented";
+                              if (aborter)
+                                aborter->print_reason(std::cerr << " (") << ')';
+                              std::cerr << '\n';
+                            }
                         }
                     }
                 };
+              missing_complement = false;
               for (unsigned i = 0; i < m; ++i)
                 {
-                  tmp(pos, comp_pos, i, 'P');
-                  tmp(neg, comp_neg, i, 'N');
+                  comp(pos, comp_pos, i, 'P');
+                  comp(neg, comp_neg, i, 'N');
                 }
             }
 
@@ -1344,7 +1425,7 @@ namespace
         }
       else
         {
-          std::cerr << "Gathering statistics..." << std::endl;
+          std::cerr << "Gathering statistics...\n";
         }
 
       spot::atomic_prop_set* ap = spot::atomic_prop_collect(f);
@@ -1362,15 +1443,41 @@ namespace
                 (*nstats)[i].product_scc.reserve(products);
               }
           }
-      for (unsigned p = 0; p < products; ++p)
+      // Decide if we need products with state-space.
+      unsigned actual_products = products;
+      if (actual_products)
+        {
+          if (missing_complement)
+            {
+              if (verbose)
+                std::cerr
+                  << ("info: complements not computed for some automata\ninfo: "
+                      "continuing with cross_checks and consistency_checks\n");
+            }
+          else if (want_stats)
+            {
+              if (verbose)
+                std::cerr
+                  << ("info: running cross_checks and consistency_checks"
+                      "just for statistics\n");
+            }
+          else
+            {
+              if (verbose)
+                std::cerr
+                  << "info: cross_checks and consistency_checks unnecessary\n";
+              actual_products = 0;
+            }
+        }
+      for (unsigned p = 0; p < actual_products; ++p)
         {
           // build a random state-space.
           spot::srand(seed);
 
           if (verbose)
-            std::cerr << "info: building state-space #" << p << '/' << products
-                      << " of " << states  << " states with seed " << seed
-                      << '\n';
+            std::cerr << "info: building state-space #" << (p+1) << '/'
+                      << products << " of " << states
+                      << " states with seed " << seed << '\n';
 
           auto statespace = spot::random_graph(states, density, ap, dict);
 
@@ -1408,7 +1515,7 @@ namespace
                       std::cerr << ("warning: not enough memory to build "
                                     "product of P") << i << " with state-space";
                       if (products > 1)
-                        std::cerr << " #" << p << '/' << products << '\n';
+                        std::cerr << " #" << (p+1) << '/' << products << '\n';
                       std::cerr << '\n';
                       ++oom_count;
                     }
@@ -1494,7 +1601,11 @@ namespace
             delete pos_map[i];
           ++seed;
         }
-      std::cerr << std::endl;
+      if (problems && quiet)
+        std::cerr << "input formula was "
+                  << bold << fstr << reset_color << "\n\n";
+      if (!quiet)
+        std::cerr << '\n';
       delete ap;
 
       // Shall we stop processing formulas now?
@@ -1615,7 +1726,7 @@ main(int argc, char** argv)
         {
           error(2, 0, "no formula to translate");
         }
-      else
+      else if (!quiet)
         {
           if (global_error_flag)
             {
@@ -1629,17 +1740,17 @@ main(int argc, char** argv)
                 err << ("error: some error was detected during the above "
                         "runs,\n       please search for 'error:' messages"
                         " in the above trace.");
-              err << std::endl;
+              err << '\n';
               end_error();
             }
           else if (timeout_count == 0
                    && ignored_exec_fail == 0 && oom_count == 0)
             {
-              std::cerr << "No problem detected." << std::endl;
+              std::cerr << "No problem detected.\n";
             }
           else
             {
-              std::cerr << "No major problem detected." << std::endl;
+              std::cerr << "No major problem detected.\n";
             }
 
           unsigned additional_errors = 0U;
@@ -1682,7 +1793,7 @@ main(int argc, char** argv)
                 }
               if (additional_errors == 1)
                 std::cerr << '.';
-              std::cerr << std::endl;
+              std::cerr << '\n';
             }
         }
 
