@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2012, 2013, 2014, 2015, 2016 Laboratoire de Recherche et
+// Copyright (C) 2012, 2013, 2014, 2015, 2016, 2018 Laboratoire de Recherche et
 // Développement de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -189,10 +189,7 @@ parse_opt(int key, char* arg, struct argp_state* as)
       spot::enable_utf8();
       break;
     case 'a':
-      opt_acc_prob = to_float(arg);
-      if (opt_acc_prob < 0.0 || opt_acc_prob > 1.0)
-        error(2, 0, "probability of acceptance set membership "
-              "should be between 0.0 and 1.0");
+      opt_acc_prob = to_probability(arg, "-a/--acc-probability");
       break;
     case 'A':
       if (looks_like_a_range(arg))
@@ -215,15 +212,13 @@ parse_opt(int key, char* arg, struct argp_state* as)
       ba_wanted = true;
       break;
     case 'e':
-      opt_density = to_float(arg);
-      if (opt_density < 0.0 || opt_density > 1.0)
-        error(2, 0, "density should be between 0.0 and 1.0");
+      opt_density = to_probability(arg, "-e/--density");
       break;
     case 'D':
       opt_deterministic = true;
       break;
     case 'n':
-      opt_automata = to_int(arg);
+      opt_automata = to_int(arg, "-n/--automata");
       break;
     case 'Q':
       opt_states = parse_range(arg);
@@ -243,7 +238,7 @@ parse_opt(int key, char* arg, struct argp_state* as)
       opt_colored = true;
       break;
     case OPT_SEED:
-      opt_seed = to_int(arg);
+      opt_seed = to_int(arg, "--seed");
       opt_seed_str = arg;
       break;
     case ARGP_KEY_ARG:
@@ -276,15 +271,13 @@ parse_opt(int key, char* arg, struct argp_state* as)
 int
 main(int argc, char** argv)
 {
-  strcpy(F_doc, "seed number");
-  strcpy(L_doc, "automaton number");
-  setup(argv);
+  return protected_main(argv, [&] {
+      strcpy(F_doc, "seed number");
+      strcpy(L_doc, "automaton number");
 
-  const argp ap = { options, parse_opt, "N|PROP...", argp_program_doc,
-                    children, nullptr, nullptr };
+      const argp ap = { options, parse_opt, "N|PROP...", argp_program_doc,
+                        children, nullptr, nullptr };
 
-  try
-    {
       // This will ensure that all objects stored in this struct are
       // destroyed before global variables.
       opt_t o;
@@ -404,9 +397,6 @@ main(int argc, char** argv)
           if (opt_automata > 0 && automaton_num >= opt_automata)
             break;
         }
-    }
-  catch (const std::runtime_error& e)
-    {
-      error(2, 0, "%s", e.what());
-    }
+      return 0;
+    });
 }
