@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2012, 2013, 2014, 2015, 2016 Laboratoire de Recherche
+// Copyright (C) 2012, 2013, 2014, 2015, 2016, 2018 Laboratoire de Recherche
 // et Développement de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -34,6 +34,7 @@
 #include "common_cout.hh"
 
 #include <sstream>
+#include <spot/tl/defaultenv.hh>
 #include <spot/tl/randomltl.hh>
 #include <spot/tl/simplify.hh>
 #include <spot/misc/random.hh>
@@ -142,7 +143,8 @@ static struct opt_t
   spot::atomic_prop_set aprops;
 }* opt;
 
-static int output = OUTPUTLTL;
+static spot::randltlgenerator::output_type output =
+  spot::randltlgenerator::LTL;
 static char* opt_pL = nullptr;
 static char* opt_pS = nullptr;
 static char* opt_pB = nullptr;
@@ -161,22 +163,22 @@ parse_opt(int key, char* arg, struct argp_state* as)
   switch (key)
     {
     case 'B':
-      output = OUTPUTBOOL;
+      output = spot::randltlgenerator::Bool;
       break;
     case 'L':
-      output = OUTPUTLTL;
+      output = spot::randltlgenerator::LTL;
       break;
     case 'n':
-      opt_formulas = to_int(arg);
+      opt_formulas = to_int(arg, "-n/--formulas");
       break;
     case 'P':
-      output = OUTPUTPSL;
+      output = spot::randltlgenerator::PSL;
       break;
     case OPT_R:
       parse_r(arg);
       break;
     case 'S':
-      output = OUTPUTSERE;
+      output = spot::randltlgenerator::SERE;
       break;
     case OPT_BOOLEAN_PRIORITIES:
       opt_pB = arg;
@@ -195,7 +197,7 @@ parse_opt(int key, char* arg, struct argp_state* as)
       opt_pS = arg;
       break;
     case OPT_SEED:
-      opt_seed = to_int(arg);
+      opt_seed = to_int(arg, "--seed");
       break;
     case OPT_TREE_SIZE:
       opt_tree_size = parse_range(arg);
@@ -235,13 +237,10 @@ parse_opt(int key, char* arg, struct argp_state* as)
 int
 main(int argc, char** argv)
 {
-  setup(argv);
+  return protected_main(argv, [&] {
+      const argp ap = { options, parse_opt, "N|PROP...", argp_program_doc,
+                        children, nullptr, nullptr };
 
-  const argp ap = { options, parse_opt, "N|PROP...", argp_program_doc,
-                    children, nullptr, nullptr };
-
-  try
-    {
       // This will ensure that all objects stored in this struct are
       // destroyed before global variables.
       opt_t o;
@@ -268,6 +267,7 @@ main(int argc, char** argv)
           opts.set("wf", opt_wf);
           opts.set("seed", opt_seed);
           opts.set("simplification_level", simplification_level);
+          opts.set("unique", opt_unique);
           return opts;
         }(), opt_pL, opt_pS, opt_pB);
 
@@ -275,23 +275,23 @@ main(int argc, char** argv)
         {
           switch (output)
             {
-            case OUTPUTLTL:
+            case spot::randltlgenerator::LTL:
               std::cout <<
                 "Use --ltl-priorities to set the following LTL priorities:\n";
               rg.dump_ltl_priorities(std::cout);
               break;
-            case OUTPUTBOOL:
+            case spot::randltlgenerator::Bool:
               std::cout <<
                 "Use --boolean-priorities to set the following Boolean "
                 "formula priorities:\n";
               rg.dump_bool_priorities(std::cout);
               break;
-            case OUTPUTPSL:
+            case spot::randltlgenerator::PSL:
               std::cout <<
                 "Use --ltl-priorities to set the following LTL priorities:\n";
               rg.dump_psl_priorities(std::cout);
               SPOT_FALLTHROUGH;
-            case OUTPUTSERE:
+            case spot::randltlgenerator::SERE:
               std::cout <<
                 "Use --sere-priorities to set the following SERE priorities:\n";
               rg.dump_sere_priorities(std::cout);
@@ -300,8 +300,6 @@ main(int argc, char** argv)
                 "formula priorities:\n";
               rg.dump_sere_bool_priorities(std::cout);
               break;
-            default:
-              error(2, 0, "internal error: unknown type of output");
             }
           exit(0);
         }
@@ -313,23 +311,14 @@ main(int argc, char** argv)
           if (!f)
             {
               error(2, 0, "failed to generate a new unique formula after %d " \
-                    "trials", MAX_TRIALS);
+                    "trials", spot::randltlgenerator::MAX_TRIALS);
             }
           else
             {
               output_formula_checked(f, nullptr, nullptr, ++count);
             }
         };
-    }
-  catch (const std::runtime_error& e)
-    {
-      error(2, 0, "%s", e.what());
-    }
-  catch (const std::invalid_argument& e)
-    {
-      error(2, 0, "%s", e.what());
-    }
-
-  flush_cout();
-  return 0;
+      flush_cout();
+      return 0;
+    });
 }
