@@ -1,6 +1,6 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2009, 2010, 2014, 2016 Laboratoire de Recherche et
-// Developpement de l'Epita
+// Copyright (C) 2009, 2010, 2014, 2016, 2018 Laboratoire de Recherche
+// et Developpement de l'Epita
 //
 // This file is part of Spot, a model checking library.
 //
@@ -17,6 +17,7 @@
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+#include "config.h"
 #include <spot/kripke/kripke.hh>
 
 namespace spot
@@ -39,7 +40,7 @@ namespace spot
   {
     // Do not assert(!done()) here.  It is OK to call
     // this function on a state without successor.
-    return 0U;
+    return {};
   }
 
   kripke::~kripke()
@@ -49,6 +50,6 @@ namespace spot
   acc_cond::mark_t
   kripke::state_acceptance_mark(const state*) const
   {
-    return 0U;
+    return {};
   }
 }
