@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2015-2018 Laboratoire de Recherche et Développement
+// Copyright (C) 2015-2019 Laboratoire de Recherche et Développement
 // de l'Epita.
 //
 // This file is part of Spot, a model checking library.
@@ -1080,7 +1080,10 @@ namespace spot
       }
 
     if (!equiv)
-      return false;
+      {
+        max = false;
+        return false;
+      }
 
     if (equiv_codes(code_, max_code))
       {
@@ -1092,6 +1095,7 @@ namespace spot
         max = false;
         return true;
       }
+    max = false;
     return false;
   }
 
@@ -1232,6 +1236,98 @@ namespace spot
     return rescode;
   }
 
+  namespace
+  {
+    template<typename Fun>
+    static void
+    top_clauses(const acc_cond::acc_code& code,
+                acc_cond::acc_op connect, acc_cond::acc_op inf_fin,
+                Fun store)
+    {
+      if (!code.empty())
+        {
+          auto pos = &code.back();
+          auto start = &code.front();
+          assert(pos - pos->sub.size == start);
+          if (pos->sub.op == connect)
+            {
+              do
+                {
+                  --pos;
+                  if (pos->sub.op == inf_fin)
+                    {
+                      for (unsigned d: pos[-1].mark.sets())
+                        {
+                          acc_cond::acc_code tmp;
+                          tmp.resize(2);
+                          tmp[0].mark = {d};
+                          tmp[1].sub.op = inf_fin;
+                          tmp[1].sub.size = 1;
+                          store(tmp);
+                        }
+                    }
+                  else
+                    {
+                      store(pos);
+                    }
+                  pos -= pos->sub.size;
+                }
+              while (pos > start);
+              return;
+            }
+          if (pos->sub.op == inf_fin)
+            {
+              for (unsigned d: pos[-1].mark.sets())
+                {
+                  acc_cond::acc_code tmp;
+                  tmp.resize(2);
+                  tmp[0].mark = {d};
+                  tmp[1].sub.op = inf_fin;
+                  tmp[1].sub.size = 1;
+                  store(tmp);
+                }
+              return;
+            }
+        }
+      store(code);
+      return;
+    }
+  }
+
+  std::vector<acc_cond::acc_code> acc_cond::acc_code::top_disjuncts() const
+  {
+    std::vector<acc_cond::acc_code> res;
+    top_clauses(*this, acc_cond::acc_op::Or, acc_cond::acc_op::Fin,
+                [&](const acc_cond::acc_code& c) { res.emplace_back(c); });
+    return res;
+  }
+
+  std::vector<acc_cond::acc_code> acc_cond::acc_code::top_conjuncts() const
+  {
+    std::vector<acc_cond::acc_code> res;
+    top_clauses(*this, acc_cond::acc_op::And, acc_cond::acc_op::Inf,
+                [&](const acc_cond::acc_code& c) { res.emplace_back(c); });
+    return res;
+  }
+
+  std::vector<acc_cond> acc_cond::top_disjuncts() const
+  {
+    std::vector<acc_cond> res;
+    top_clauses(code_, acc_cond::acc_op::Or, acc_cond::acc_op::Fin,
+                [&](const acc_cond::acc_code& c)
+                { res.emplace_back(num_, c); });
+    return res;
+  }
+
+  std::vector<acc_cond> acc_cond::top_conjuncts() const
+  {
+    std::vector<acc_cond> res;
+    top_clauses(code_, acc_cond::acc_op::And, acc_cond::acc_op::Inf,
+                [&](const acc_cond::acc_code& c)
+                { res.emplace_back(num_, c); });
+    return res;
+  }
+
   std::pair<bool, acc_cond::mark_t>
   acc_cond::sat_unsat_mark(bool sat) const
   {
