@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2011, 2013-2018  Laboratoire de Recherche et
+// Copyright (C) 2011, 2013-2019  Laboratoire de Recherche et
 // Développement de l'Epita (LRDE).
 // Copyright (C) 2004, 2005  Laboratoire d'Informatique de Paris 6 (LIP6),
 // département Systèmes Répartis Coopératifs (SRC), Université Pierre
@@ -563,9 +563,8 @@ namespace spot
   emptiness_check_ptr
   explicit_tau03_opt_search(const const_twa_ptr& a, option_map o)
   {
-    return
-      std::make_shared<tau03_opt_search<explicit_tau03_opt_search_heap>>(a,
-                                                                         0, o);
+    return SPOT_make_shared_enabled__
+      (tau03_opt_search<explicit_tau03_opt_search_heap>, a, 0, o);
   }
 
 }
